@@ -1,7 +1,7 @@
 /* dns_query.h - Execute outgoing dns queries and write entries to cache
 
    Copyright (C) 2000, 2001 Thomas Moestl
-   Copyright (C) 2002, 2003, 2004 Paul A. Rombouts
+   Copyright (C) 2002, 2003, 2004, 2006 Paul A. Rombouts
 
 This file is part of the pdnsd package.
 
@@ -30,11 +30,11 @@ Boston, MA 02111-1307, USA.  */
 typedef struct qhintnode_s qhintnode_t;
 
 /* --- parallel query */
-int r_dns_cached_resolve(unsigned char *name, dns_cent_t **cachedp,
-			 int hops, qhintnode_t *qhlist, int thint, time_t queryts,
+int r_dns_cached_resolve(unsigned char *name, int thint, dns_cent_t **cachedp,
+			 int hops, qhintnode_t *qhlist, time_t queryts,
 			 unsigned char *c_soa);
-#define dns_cached_resolve(name,cachedp,hops,thint,queryts,c_soa) \
-        r_dns_cached_resolve(name,cachedp,hops,NULL,thint,queryts,c_soa)
+#define dns_cached_resolve(name,thint,cachedp,hops,queryts,c_soa) \
+        r_dns_cached_resolve(name,thint,cachedp,hops,NULL,queryts,c_soa)
 
 int query_uptest(pdnsd_a *addr, int port, time_t timeout, int rep);
 
